@@ -18,6 +18,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from enum import Enum
 from logging import getLogger
 from ssl import SSLSocket
 
@@ -52,10 +53,57 @@ from neo4j.io._common import (
 log = getLogger("neo4j")
 
 
+class ServerStates(Enum):
+    CONNECTED = "CONNECTED"
+    READY = "READY"
+    STREAMING = "STREAMING"
+    TX_READY_OR_TX_STREAMING = "TX_READY||TX_STREAMING"
+    FAILED = "FAILED"
+
+
+class ServerStateManager:
+    _STATE_TRANSITIONS = {
+        ServerStates.CONNECTED: {
+            "hello": ServerStates.READY,
+        },
+        ServerStates.READY: {
+            "run": ServerStates.STREAMING,
+            "begin": ServerStates.TX_READY_OR_TX_STREAMING,
+        },
+        ServerStates.STREAMING: {
+            "pull": ServerStates.READY,
+            "discard": ServerStates.READY,
+            "reset": ServerStates.READY,
+        },
+        ServerStates.TX_READY_OR_TX_STREAMING: {
+            "commit": ServerStates.READY,
+            "rollback": ServerStates.READY,
+            "reset": ServerStates.READY,
+        },
+        ServerStates.FAILED: {
+            "reset": ServerStates.READY,
+        }
+    }
+
+    def __init__(self, init_state, on_change=None):
+        self.state = init_state
+        self._on_change = on_change
+
+    def transition(self, message, metadata):
+        if metadata.get("has_more"):
+            return
+        state_before = self.state
+        self.state = self._STATE_TRANSITIONS\
+            .get(self.state, {})\
+            .get(message, self.state)
+        if state_before != self.state and callable(self._on_change):
+            self._on_change(state_before, self.state)
+
+
 class Bolt3(Bolt):
     """ Protocol handler for Bolt 3.
 
-    This is supported by Neo4j versions 3.5, 4.0, 4.1 and 4.2.
+    This is supported by Neo4j versions 3.5, 4.0, 4.1, 4.2, 4.3, and 4.4.
     """
 
     PROTOCOL_VERSION = Version(3, 0)
@@ -64,6 +112,26 @@ class Bolt3(Bolt):
 
     supports_multiple_databases = False
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._server_state_manager = ServerStateManager(
+            ServerStates.CONNECTED, on_change=self._on_server_state_change
+        )
+
+    def _on_server_state_change(self, old_state, new_state):
+        log.debug("[#%04X]  State: %s > %s", self.local_port,
+                  old_state.name, new_state.name)
+
+    @property
+    def is_reset(self):
+        # We can't be sure of the server's state if there are still pending
+        # responses. Unless the last message we sent was RESET. In that case
+        # the server state will always be READY when we're done.
+        if (self.responses and self.responses[-1]
+                and self.responses[-1].message == "reset"):
+            return True
+        return self._server_state_manager.state == ServerStates.READY
+
     @property
     def encrypted(self):
         return isinstance(self.socket, SSLSocket)
@@ -72,13 +140,6 @@ class Bolt3(Bolt):
     def der_encoded_server_certificate(self):
         return self.socket.getpeercert(binary_form=True)
 
-    @property
-    def local_port(self):
-        try:
-            return self.socket.getsockname()[1]
-        except OSError:
-            return 0
-
     def get_base_headers(self):
         return {
             "user_agent": self.user_agent,
@@ -92,28 +153,32 @@ class Bolt3(Bolt):
             logged_headers["credentials"] = "*******"
         log.debug("[#%04X]  C: HELLO %r", self.local_port, logged_headers)
         self._append(b"\x01", (headers,),
-                     response=InitResponse(self, on_success=self.server_info.update))
+                     response=InitResponse(self, "hello",
+                                           on_success=self.server_info.update))
         self.send_all()
         self.fetch_all()
         check_supported_server_product(self.server_info.agent)
 
-    def route(self, database=None, bookmarks=None):
-        if database is not None:  # default database
-            raise ConfigurationError("Database name parameter for selecting database is not "
-                                     "supported in Bolt Protocol {!r}. Database name {!r}. "
-                                     "Server Agent {!r}.".format(Bolt3.PROTOCOL_VERSION, database,
-                                                                 self.server_info.agent))
+    def route(self, database=None, imp_user=None, bookmarks=None):
+        if database is not None:
+            raise ConfigurationError(
+                "Database name parameter for selecting database is not "
+                "supported in Bolt Protocol {!r}. Database name {!r}. "
+                "Server Agent {!r}".format(
+                    self.PROTOCOL_VERSION, database, self.server_info.agent
+                )
+            )
+        if imp_user is not None:
+            raise ConfigurationError(
+                "Impersonation is not supported in Bolt Protocol {!r}. "
+                "Trying to impersonate {!r}.".format(
+                    self.PROTOCOL_VERSION, imp_user
+                )
+            )
 
         metadata = {}
         records = []
 
-        def fail(md):
-            from neo4j._exceptions import BoltRoutingError
-            if md.get("code") == "Neo.ClientError.Procedure.ProcedureNotFound":
-                raise BoltRoutingError("Server does not support routing", self.unresolved_address)
-            else:
-                raise BoltRoutingError("Routing support broken on server", self.unresolved_address)
-
         # Ignoring database and bookmarks because there is no multi-db support.
         # The bookmarks are only relevant for making sure a previously created
         # db exists before querying a routing table for it.
@@ -121,7 +186,7 @@ class Bolt3(Bolt):
             "CALL dbms.cluster.routing.getRoutingTable($context)",  # This is an internal procedure call. Only available if the Neo4j 3.5 is setup with clustering.
             {"context": self.routing_context},
             mode="r",                                               # Bolt Protocol Version(3, 0) supports mode="r"
-            on_success=metadata.update, on_failure=fail
+            on_success=metadata.update
         )
         self.pull(on_success=metadata.update, on_records=records.extend)
         self.send_all()
@@ -129,9 +194,22 @@ class Bolt3(Bolt):
         routing_info = [dict(zip(metadata.get("fields", ()), values)) for values in records]
         return routing_info
 
-    def run(self, query, parameters=None, mode=None, bookmarks=None, metadata=None, timeout=None, db=None, **handlers):
+    def run(self, query, parameters=None, mode=None, bookmarks=None,
+            metadata=None, timeout=None, db=None, imp_user=None, **handlers):
         if db is not None:
-            raise ConfigurationError("Database name parameter for selecting database is not supported in Bolt Protocol {!r}. Database name {!r}.".format(Bolt3.PROTOCOL_VERSION, db))
+            raise ConfigurationError(
+                "Database name parameter for selecting database is not "
+                "supported in Bolt Protocol {!r}. Database name {!r}.".format(
+                    self.PROTOCOL_VERSION, db
+                )
+            )
+        if imp_user is not None:
+            raise ConfigurationError(
+                "Impersonation is not supported in Bolt Protocol {!r}. "
+                "Trying to impersonate {!r}.".format(
+                    self.PROTOCOL_VERSION, imp_user
+                )
+            )
         if not parameters:
             parameters = {}
         extra = {}
@@ -155,25 +233,37 @@ class Bolt3(Bolt):
         fields = (query, parameters, extra)
         log.debug("[#%04X]  C: RUN %s", self.local_port, " ".join(map(repr, fields)))
         if query.upper() == u"COMMIT":
-            self._append(b"\x10", fields, CommitResponse(self, **handlers))
+            self._append(b"\x10", fields, CommitResponse(self, "run",
+                                                         **handlers))
         else:
-            self._append(b"\x10", fields, Response(self, **handlers))
-        self._is_reset = False
+            self._append(b"\x10", fields, Response(self, "run", **handlers))
 
     def discard(self, n=-1, qid=-1, **handlers):
         # Just ignore n and qid, it is not supported in the Bolt 3 Protocol.
         log.debug("[#%04X]  C: DISCARD_ALL", self.local_port)
-        self._append(b"\x2F", (), Response(self, **handlers))
+        self._append(b"\x2F", (), Response(self, "discard", **handlers))
 
     def pull(self, n=-1, qid=-1, **handlers):
         # Just ignore n and qid, it is not supported in the Bolt 3 Protocol.
         log.debug("[#%04X]  C: PULL_ALL", self.local_port)
-        self._append(b"\x3F", (), Response(self, **handlers))
-        self._is_reset = False
+        self._append(b"\x3F", (), Response(self, "pull", **handlers))
 
-    def begin(self, mode=None, bookmarks=None, metadata=None, timeout=None, db=None, **handlers):
+    def begin(self, mode=None, bookmarks=None, metadata=None, timeout=None,
+              db=None, imp_user=None, **handlers):
         if db is not None:
-            raise ConfigurationError("Database name parameter for selecting database is not supported in Bolt Protocol {!r}. Database name {!r}.".format(Bolt3.PROTOCOL_VERSION, db))
+            raise ConfigurationError(
+                "Database name parameter for selecting database is not "
+                "supported in Bolt Protocol {!r}. Database name {!r}.".format(
+                    self.PROTOCOL_VERSION, db
+                )
+            )
+        if imp_user is not None:
+            raise ConfigurationError(
+                "Impersonation is not supported in Bolt Protocol {!r}. "
+                "Trying to impersonate {!r}.".format(
+                    self.PROTOCOL_VERSION, imp_user
+                )
+            )
         extra = {}
         if mode in (READ_ACCESS, "r"):
             extra["mode"] = "r"  # It will default to mode "w" if nothing is specified
@@ -193,16 +283,15 @@ class Bolt3(Bolt):
             except TypeError:
                 raise TypeError("Timeout must be specified as a number of seconds")
         log.debug("[#%04X]  C: BEGIN %r", self.local_port, extra)
-        self._append(b"\x11", (extra,), Response(self, **handlers))
-        self._is_reset = False
+        self._append(b"\x11", (extra,), Response(self, "begin", **handlers))
 
     def commit(self, **handlers):
         log.debug("[#%04X]  C: COMMIT", self.local_port)
-        self._append(b"\x12", (), CommitResponse(self, **handlers))
+        self._append(b"\x12", (), CommitResponse(self, "commit", **handlers))
 
     def rollback(self, **handlers):
         log.debug("[#%04X]  C: ROLLBACK", self.local_port)
-        self._append(b"\x13", (), Response(self, **handlers))
+        self._append(b"\x13", (), Response(self, "rollback", **handlers))
 
     def reset(self):
         """ Add a RESET message to the outgoing queue, send
@@ -213,10 +302,13 @@ class Bolt3(Bolt):
             raise BoltProtocolError("RESET failed %r" % metadata, address=self.unresolved_address)
 
         log.debug("[#%04X]  C: RESET", self.local_port)
-        self._append(b"\x0F", response=Response(self, on_failure=fail))
+        self._append(b"\x0F", response=Response(self, "reset", on_failure=fail))
         self.send_all()
         self.fetch_all()
-        self._is_reset = True
+
+    def goodbye(self):
+        log.debug("[#%04X]  C: GOODBYE", self.local_port)
+        self._append(b"\x02", ())
 
     def fetch_message(self):
         """ Receive at most one message from the server, if available.
@@ -249,12 +341,15 @@ class Bolt3(Bolt):
         response.complete = True
         if summary_signature == b"\x70":
             log.debug("[#%04X]  S: SUCCESS %r", self.local_port, summary_metadata)
+            self._server_state_manager.transition(response.message,
+                                                  summary_metadata)
             response.on_success(summary_metadata or {})
         elif summary_signature == b"\x7E":
             log.debug("[#%04X]  S: IGNORED", self.local_port)
             response.on_ignored(summary_metadata or {})
         elif summary_signature == b"\x7F":
             log.debug("[#%04X]  S: FAILURE %r", self.local_port, summary_metadata)
+            self._server_state_manager.state = ServerStates.FAILED
             try:
                 response.on_failure(summary_metadata or {})
             except (ServiceUnavailable, DatabaseUnavailable):
