@@ -22,9 +22,9 @@
 from neo4j.work.result import Result
 from neo4j.data import DataHydrator
 from neo4j.exceptions import (
-    IncompleteCommit,
     TransactionError,
 )
+from neo4j.io import ConnectionErrorHandler
 
 
 class Transaction:
@@ -38,14 +38,18 @@ class Transaction:
 
     """
 
-    def __init__(self, connection, fetch_size, on_closed, on_network_error):
+    def __init__(self, connection, fetch_size, on_closed, on_error):
         self._connection = connection
+        self._error_handling_connection = ConnectionErrorHandler(
+            connection, self._error_handler
+        )
         self._bookmark = None
         self._results = []
         self._closed = False
+        self._last_error = None
         self._fetch_size = fetch_size
         self._on_closed = on_closed
-        self._on_network_error = on_network_error
+        self._on_error = on_error
 
     def __enter__(self):
         return self
@@ -58,16 +62,20 @@ class Transaction:
             self.commit()
         self.close()
 
-    def _begin(self, database, bookmarks, access_mode, metadata, timeout):
-        self._connection.begin(bookmarks=bookmarks, metadata=metadata,
-                               timeout=timeout, mode=access_mode, db=database)
+    def _begin(self, database, imp_user, bookmarks, access_mode, metadata, timeout):
+        self._connection.begin(
+            bookmarks=bookmarks, metadata=metadata, timeout=timeout,
+            mode=access_mode, db=database, imp_user=imp_user
+        )
+        self._error_handling_connection.send_all()
+        self._error_handling_connection.fetch_all()
 
     def _result_on_closed_handler(self):
         pass
 
-    def _result_on_network_error_handler(self):
-        self._closed = True
-        self._on_network_error()
+    def _error_handler(self, exc):
+        self._last_error = exc
+        self._on_error(exc)
 
     def _consume_results(self):
         for result in self._results:
@@ -107,7 +115,10 @@ class Transaction:
             raise ValueError("Query object is only supported for session.run")
 
         if self._closed:
-            raise TransactionError("Transaction closed")
+            raise TransactionError(self, "Transaction closed")
+        if self._last_error:
+            raise TransactionError(self,
+                                   "Transaction failed") from self._last_error
 
         if (self._results
                 and self._connection.supports_multiple_results is False):
@@ -116,10 +127,14 @@ class Transaction:
             # have any qid to fetch in batches.
             self._results[-1]._buffer_all()
 
+        hydrant = DataHydrator(
+            patch_utc="utc" in self._connection.bolt_patches
+        )
+
         result = Result(
-            self._connection, DataHydrator(), self._fetch_size,
+            self._connection, hydrant, self._fetch_size,
             self._result_on_closed_handler,
-            self._result_on_network_error_handler
+            self._error_handler
         )
         self._results.append(result)
 
@@ -133,7 +148,11 @@ class Transaction:
         :raise TransactionError: if the transaction is already closed
         """
         if self._closed:
-            raise TransactionError("Transaction closed")
+            raise TransactionError(self, "Transaction closed")
+        if self._last_error:
+            raise TransactionError(self,
+                                   "Transaction failed") from self._last_error
+
         metadata = {}
         try:
             self._consume_results()  # DISCARD pending records then do a commit.
@@ -153,10 +172,13 @@ class Transaction:
         :raise TransactionError: if the transaction is already closed
         """
         if self._closed:
-            raise TransactionError("Transaction closed")
+            raise TransactionError(self, "Transaction closed")
+
         metadata = {}
         try:
-            if not self._connection.is_reset:
+            if not (self._connection.defunct()
+                    or self._connection.closed()
+                    or self._connection.is_reset):
                 # DISCARD pending records then do a rollback.
                 self._consume_results()
                 self._connection.rollback(on_success=metadata.update)
